import sys
from logging import StreamHandler

from ..formatters.json import JSONFormatter


class StreamTransport(StreamHandler):
    def __init__(self, **kwargs):
        level = kwargs.get("level")
        StreamHandler.__init__(self)

        self.setLevel(level)
        self.setFormatter(JSONFormatter())

    def emit(self, record):
        message = self.format(record)
        return sys.stdout.write(message + "\n")



"""
    Logging module which implements MEH standards.
"""
import logging
import traceback

from .utils import package
from .enums.levels import Levels
from .transports.stream import StreamTransport

# Defaults
DEFAULT_MIN_LEVEL = Levels.NOT_SET.value
DEFAULT_MAX_LEVEL = Levels.MAX_LEVEL.value


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, "_instance"):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls, *args, **kw)
        return cls._instance


class Logger(Singleton):
    def __init__(self, name=package.app_name):
        self.name = name

        logging.root.setLevel(DEFAULT_MIN_LEVEL)

        self.logger = logging.getLogger(name)
        self.add_transport(StreamTransport(level=Levels.DEBUG.value))

        if not name or name == package.FALLBACK_PACKAGE_NAME:
            self.warning("No app name was found! A `package.json` file containing the app name is required in the root of the project (cwd)")

    def __str__(self):
        return f"<Logger name={self.name}>"

    def set_level(self, level):
        self.logger.setLevel(level)

        for handler in self.logger.handlers:
            handler.setLevel(level)

        return self

    def __handle_stack(self, message):
        if isinstance(message, Exception):
            return {
                "stack": "".join(
                    traceback.TracebackException.from_exception(message).format()
                ),
            }

        return {"stack": None}

    def add_transport(self, handler):
        self.logger.addHandler(handler)
        return self

    def debug(self, message):
        return self.logger.log(
            Levels.DEBUG.value, message, extra=self.__handle_stack(message)
        )

    def info(self, message):
        return self.logger.log(
            Levels.INFO.value, message, extra=self.__handle_stack(message)
        )

    def warning(self, message):
        return self.logger.log(
            Levels.WARNING.value, message, extra=self.__handle_stack(message)
        )

    def error(self, message):
        return self.logger.log(
            Levels.ERROR.value, message, extra=self.__handle_stack(message)
        )


logger = Logger()



"""
"""
import os
import json

# Package.json consts
PACKAGE_FILE_NAME = "package.json"
PACKAGE_LOCATION = os.path.join(os.getcwd(), PACKAGE_FILE_NAME)
ENCODING = "utf-8"


def read_package_file():
    try:
        file = open(PACKAGE_LOCATION, encoding=ENCODING)
        return json.load(file)
    except Exception:
        return {}



import logging
from enum import Enum


class Levels(Enum):
    NOT_SET = logging.NOTSET
    DEBUG = logging.DEBUG
    INFO = logging.INFO
    WARNING = logging.WARNING
    WARN = WARNING
    ERROR = logging.ERROR
    CRITICAL = logging.CRITICAL
    MAX_LEVEL = logging.CRITICAL + 10



import json

from logging import Formatter
from ..utils.package import JSON_DEFAULTS


class JSONFormatter(Formatter):
    def __init__(self):
        super().__init__()

    def __format(self, record):
        message = record.message
        level = record.levelname
        obj = {"level": level.lower(), "message": str(message)} | JSON_DEFAULTS

        if hasattr(record, "stack") and record.stack is not None:
            obj["stack"] = record.stack

        return json.dumps(obj)

    def formatMessage(self, record):
        return self.__format(record)



"""
Generate the app defaults
"""
import platform
import os
import re

from datetime import datetime
from .read_package import read_package_file

# Platform consts
PLATFORM = "python"
PLATFORM_VERSION = platform.python_version()

# Fallbacks
FALLBACK_PACKAGE_NAME = "unknown-app"
FALLBACK_PACKAGE_VERSION = "0.0.0"

PYTHON_ENV = os.getenv("ENV", os.getenv("PYTHON_ENV", ""))

def parse_app_name(name):
    if PYTHON_ENV == "acceptance":
        return re.sub("-acc$", "", name)
    elif PYTHON_ENV == "test":
        return re.sub("-test$", "", name)

    return name

package = read_package_file()

app_name = package.get("name", FALLBACK_PACKAGE_NAME)
app_name = parse_app_name(app_name)
app_version = package.get("version", FALLBACK_PACKAGE_VERSION)

JSON_DEFAULTS = {
    "appName": app_name,
    "appVersion": app_version,
    "platform": PLATFORM,
    "platformVersion": PLATFORM_VERSION,
    "timestamp": datetime.now().astimezone().isoformat(),
}
